@@ -1,6 +1,7 @@
 /* Author: Jason Tang	  <jtang@tresys.com>
+ *         Christopher Ashworth <cashworth@tresys.com>
  *
- * Copyright (C) 2004-2005 Tresys Technology, LLC
+ * Copyright (C) 2004-2006 Tresys Technology, LLC
  * Copyright (C) 2005 Red Hat, Inc.
  * 
  *  This library is free software; you can redistribute it and/or
@@ -48,19 +49,20 @@
 #include "database_policydb.h"
 #include "policy.h"
 
-static void semanage_direct_destroy(semanage_handle_t *sh);
-static int semanage_direct_disconnect(semanage_handle_t *sh);
-static int semanage_direct_begintrans(semanage_handle_t *sh);
-static int semanage_direct_commit(semanage_handle_t *sh);
-static int semanage_direct_install(semanage_handle_t *sh, char *data, size_t data_len);
-static int semanage_direct_upgrade(semanage_handle_t *sh,
-				   char *data, size_t data_len);
-static int semanage_direct_install_base(semanage_handle_t *sh,
-					char *base_data, size_t data_len);
-static int semanage_direct_remove(semanage_handle_t *sh,
-				  char *module_name);
-static int semanage_direct_list(semanage_handle_t *sh,
-				semanage_module_info_t **modinfo, int *num_modules);
+static void semanage_direct_destroy(semanage_handle_t * sh);
+static int semanage_direct_disconnect(semanage_handle_t * sh);
+static int semanage_direct_begintrans(semanage_handle_t * sh);
+static int semanage_direct_commit(semanage_handle_t * sh);
+static int semanage_direct_install(semanage_handle_t * sh, char *data,
+				   size_t data_len);
+static int semanage_direct_upgrade(semanage_handle_t * sh, char *data,
+				   size_t data_len);
+static int semanage_direct_install_base(semanage_handle_t * sh, char *base_data,
+					size_t data_len);
+static int semanage_direct_remove(semanage_handle_t * sh, char *module_name);
+static int semanage_direct_list(semanage_handle_t * sh,
+				semanage_module_info_t ** modinfo,
+				int *num_modules);
 
 static struct semanage_policy_table direct_funcs = {
 	.get_serial = semanage_get_commit_number,
@@ -75,31 +77,35 @@ static struct semanage_policy_table direct_funcs = {
 	.list = semanage_direct_list
 };
 
-int semanage_direct_is_managed(semanage_handle_t *sh) {
+int semanage_direct_is_managed(semanage_handle_t * sh)
+{
 	char polpath[PATH_MAX];
 
-	snprintf(polpath, PATH_MAX, "%s%s", selinux_path(), sh->conf->store_path);
-	
+	snprintf(polpath, PATH_MAX, "%s%s", selinux_path(),
+		 sh->conf->store_path);
+
 	if (semanage_check_init(polpath))
 		goto err;
 
-	if (semanage_access_check(sh) < 0) 
+	if (semanage_access_check(sh) < 0)
 		return 0;
 
 	return 1;
 
-	err:
+      err:
 	ERR(sh, "could not check whether policy is managed");
-	return STATUS_ERR;	
+	return STATUS_ERR;
 }
 
 /* Check that the module store exists, creating it if necessary.
  */
-int semanage_direct_connect(semanage_handle_t *sh) {
+int semanage_direct_connect(semanage_handle_t * sh)
+{
 	char polpath[PATH_MAX];
 
-	snprintf(polpath, PATH_MAX, "%s%s", selinux_path(), sh->conf->store_path);
-	
+	snprintf(polpath, PATH_MAX, "%s%s", selinux_path(),
+		 sh->conf->store_path);
+
 	if (semanage_check_init(polpath))
 		goto err;
 
@@ -107,7 +113,7 @@ int semanage_direct_connect(semanage_handle_t *sh) {
 		if (semanage_create_store(sh, 1))
 			goto err;
 
-	if (semanage_access_check(sh) < SEMANAGE_CAN_READ) 
+	if (semanage_access_check(sh) < SEMANAGE_CAN_READ)
 		goto err;
 
 	sh->u.direct.translock_file_fd = -1;
@@ -116,67 +122,70 @@ int semanage_direct_connect(semanage_handle_t *sh) {
 	/* set up function pointers */
 	sh->funcs = &direct_funcs;
 
-        /* Object databases: local modifications */
-	if (user_base_file_dbase_init(sh, 
-		semanage_fname(SEMANAGE_USERS_BASE_LOCAL),
-		semanage_user_base_dbase_local(sh)) < 0)
+	/* Object databases: local modifications */
+	if (user_base_file_dbase_init(sh,
+				      semanage_fname(SEMANAGE_USERS_BASE_LOCAL),
+				      semanage_user_base_dbase_local(sh)) < 0)
 		goto err;
 
-	if (user_extra_file_dbase_init(sh, 
-		semanage_fname(SEMANAGE_USERS_EXTRA_LOCAL),
-		semanage_user_extra_dbase_local(sh)) < 0)
+	if (user_extra_file_dbase_init(sh,
+				       semanage_fname
+				       (SEMANAGE_USERS_EXTRA_LOCAL),
+				       semanage_user_extra_dbase_local(sh)) < 0)
 		goto err;
 
 	if (user_join_dbase_init(sh,
-		semanage_user_base_dbase_local(sh),
-		semanage_user_extra_dbase_local(sh),
-		semanage_user_dbase_local(sh)) < 0)
+				 semanage_user_base_dbase_local(sh),
+				 semanage_user_extra_dbase_local(sh),
+				 semanage_user_dbase_local(sh)) < 0)
 		goto err;
 
-	if (port_file_dbase_init(sh, 
-		semanage_fname(SEMANAGE_PORTS_LOCAL), 
-		semanage_port_dbase_local(sh)) < 0)
+	if (port_file_dbase_init(sh,
+				 semanage_fname(SEMANAGE_PORTS_LOCAL),
+				 semanage_port_dbase_local(sh)) < 0)
 		goto err;
 
-	if (iface_file_dbase_init(sh, 
-		semanage_fname(SEMANAGE_INTERFACES_LOCAL),
-		semanage_iface_dbase_local(sh)) < 0)
+	if (iface_file_dbase_init(sh,
+				  semanage_fname(SEMANAGE_INTERFACES_LOCAL),
+				  semanage_iface_dbase_local(sh)) < 0)
 		goto err;
 
-	if (bool_file_dbase_init(sh, 
-		semanage_fname(SEMANAGE_BOOLEANS_LOCAL),
-		semanage_bool_dbase_local(sh)) < 0)
+	if (bool_file_dbase_init(sh,
+				 semanage_fname(SEMANAGE_BOOLEANS_LOCAL),
+				 semanage_bool_dbase_local(sh)) < 0)
 		goto err;
 
-	if (fcontext_file_dbase_init(sh, 
-		semanage_fname(SEMANAGE_FC_LOCAL), 
-		semanage_fcontext_dbase_local(sh)) < 0)
+	if (fcontext_file_dbase_init(sh,
+				     semanage_fname(SEMANAGE_FC_LOCAL),
+				     semanage_fcontext_dbase_local(sh)) < 0)
 		goto err;
 
-	if (seuser_file_dbase_init(sh, 
-		semanage_fname(SEMANAGE_SEUSERS_LOCAL),
-		semanage_seuser_dbase_local(sh)) < 0)
+	if (seuser_file_dbase_init(sh,
+				   semanage_fname(SEMANAGE_SEUSERS_LOCAL),
+				   semanage_seuser_dbase_local(sh)) < 0)
 		goto err;
 
-	if (node_file_dbase_init(sh, 
-		semanage_fname(SEMANAGE_NODES_LOCAL),
-		semanage_node_dbase_local(sh)) < 0)
+	if (node_file_dbase_init(sh,
+				 semanage_fname(SEMANAGE_NODES_LOCAL),
+				 semanage_node_dbase_local(sh)) < 0)
 		goto err;
 
 	/* Object databases: local modifications + policy */
-	if (user_base_policydb_dbase_init(sh, 
-		semanage_user_base_dbase_policy(sh)) < 0)
+	if (user_base_policydb_dbase_init(sh,
+					  semanage_user_base_dbase_policy(sh)) <
+	    0)
 		goto err;
 
-	if (user_extra_file_dbase_init(sh, 
-		semanage_fname(SEMANAGE_USERS_EXTRA), 
-		semanage_user_extra_dbase_policy(sh)) < 0)
+	if (user_extra_file_dbase_init(sh,
+				       semanage_fname(SEMANAGE_USERS_EXTRA),
+				       semanage_user_extra_dbase_policy(sh)) <
+	    0)
 		goto err;
 
-	if (user_join_dbase_init(sh, 
-		semanage_user_base_dbase_policy(sh),
-		semanage_user_extra_dbase_policy(sh),
-		semanage_user_dbase_policy(sh)) < 0)
+	if (user_join_dbase_init(sh,
+				 semanage_user_base_dbase_policy(sh),
+				 semanage_user_extra_dbase_policy(sh),
+				 semanage_user_dbase_policy(sh)) < 0)
 		goto err;
 
 	if (port_policydb_dbase_init(sh, semanage_port_dbase_policy(sh)) < 0)
@@ -188,14 +197,14 @@ int semanage_direct_connect(semanage_handle_t *sh) {
 	if (bool_policydb_dbase_init(sh, semanage_bool_dbase_policy(sh)) < 0)
 		goto err;
 
-	if (fcontext_file_dbase_init(sh, 
-		semanage_fname(SEMANAGE_FC), 
-		semanage_fcontext_dbase_policy(sh)) < 0)
+	if (fcontext_file_dbase_init(sh,
+				     semanage_fname(SEMANAGE_FC),
+				     semanage_fcontext_dbase_policy(sh)) < 0)
 		goto err;
 
-	if (seuser_file_dbase_init(sh, 
-		semanage_fname(SEMANAGE_SEUSERS), 
-		semanage_seuser_dbase_policy(sh)) < 0)
+	if (seuser_file_dbase_init(sh,
+				   semanage_fname(SEMANAGE_SEUSERS),
+				   semanage_seuser_dbase_policy(sh)) < 0)
 		goto err;
 
 	if (node_policydb_dbase_init(sh, semanage_node_dbase_policy(sh)) < 0)
@@ -207,23 +216,26 @@ int semanage_direct_connect(semanage_handle_t *sh) {
 
 	return STATUS_SUCCESS;
 
-	err:
+      err:
 	ERR(sh, "could not establish direct connection");
-	sepol_handle_destroy(sh->sepolh);
 	return STATUS_ERR;
 }
 
-static void semanage_direct_destroy(semanage_handle_t *sh) {
+static void semanage_direct_destroy(semanage_handle_t * sh)
+{
 	/* do nothing */
 	sh = NULL;
 }
 
-static int semanage_direct_disconnect(semanage_handle_t *sh) {
+static int semanage_direct_disconnect(semanage_handle_t * sh)
+{
 	/* destroy transaction */
 	if (sh->is_in_transaction) {
 		/* destroy sandbox */
-		if (semanage_remove_directory(semanage_path(SEMANAGE_TMP, SEMANAGE_TOPLEVEL)) < 0) {
-			ERR(sh, "Could not cleanly remove sandbox %s.", semanage_path(SEMANAGE_TMP, SEMANAGE_TOPLEVEL));
+		if (semanage_remove_directory
+		    (semanage_path(SEMANAGE_TMP, SEMANAGE_TOPLEVEL)) < 0) {
+			ERR(sh, "Could not cleanly remove sandbox %s.",
+			    semanage_path(SEMANAGE_TMP, SEMANAGE_TOPLEVEL));
 			return -1;
 		}
 		semanage_release_trans_lock(sh);
@@ -257,8 +269,9 @@ static int semanage_direct_disconnect(semanage_handle_t *sh) {
 	return 0;
 }
 
-static int semanage_direct_begintrans(semanage_handle_t *sh) {
-	
+static int semanage_direct_begintrans(semanage_handle_t * sh)
+{
+
 	if (semanage_access_check(sh) != SEMANAGE_CAN_WRITE) {
 		return -1;
 	}
@@ -281,14 +294,15 @@ static int semanage_direct_begintrans(semanage_handle_t *sh) {
  * set to NULL upon entering this function.  Returns 0 on success, -1
  * if out of memory, or -2 if data did not represent a module.
  */
-static int parse_module_headers(semanage_handle_t *sh, char *module_data,
+static int parse_module_headers(semanage_handle_t * sh, char *module_data,
 				size_t data_len, char **module_name,
-				char **version, char **filename) {
+				char **version, char **filename)
+{
 	struct sepol_policy_file *pf;
 	int file_type;
 	const char *module_path;
 	*module_name = *version = *filename = NULL;
-	
+
 	if (sepol_policy_file_create(&pf)) {
 		ERR(sh, "Out of memory!");
 		return -1;
@@ -306,12 +320,14 @@ static int parse_module_headers(semanage_handle_t *sh, char *module_data,
 	sepol_policy_file_free(pf);
 	if (file_type != SEPOL_POLICY_MOD) {
 		if (file_type == SEPOL_POLICY_BASE)
-			ERR(sh, "Received a base module, expected a non-base module.");
+			ERR(sh,
+			    "Received a base module, expected a non-base module.");
 		else
 			ERR(sh, "Data did not represent a module.");
 		return -2;
 	}
-	if ((module_path = semanage_path(SEMANAGE_TMP, SEMANAGE_MODULES)) == NULL) {
+	if ((module_path =
+	     semanage_path(SEMANAGE_TMP, SEMANAGE_MODULES)) == NULL) {
 		return -1;
 	}
 	if (asprintf(filename, "%s/%s.pp", module_path, *module_name) == -1) {
@@ -325,8 +341,9 @@ static int parse_module_headers(semanage_handle_t *sh, char *module_data,
  * Returns 0 on success, -1 if out of memory, or -2 if data did not
  * represent a module.
  */
-static int parse_base_headers(semanage_handle_t *sh,
-			      char *module_data, size_t data_len) {
+static int parse_base_headers(semanage_handle_t * sh,
+			      char *module_data, size_t data_len)
+{
 	struct sepol_policy_file *pf;
 	char *module_name = NULL, *version = NULL;
 	int file_type;
@@ -350,7 +367,8 @@ static int parse_base_headers(semanage_handle_t *sh,
 	free(version);
 	if (file_type != SEPOL_POLICY_BASE) {
 		if (file_type == SEPOL_POLICY_MOD)
-			ERR(sh, "Received a non-base module, expected a base module.");
+			ERR(sh,
+			    "Received a non-base module, expected a base module.");
 		else
 			ERR(sh, "Data did not represent a module.");
 		return -2;
@@ -360,10 +378,13 @@ static int parse_base_headers(semanage_handle_t *sh,
 
 /* Writes a block of data to a file.  Returns 0 on success, -1 on
  * error. */
-static int write_file(semanage_handle_t *sh,
-		      const char *filename, char *data, size_t num_bytes) {
+static int write_file(semanage_handle_t * sh,
+		      const char *filename, char *data, size_t num_bytes)
+{
 	int out;
-	if ((out = open(filename, O_WRONLY | O_CREAT | O_TRUNC, S_IRUSR | S_IWUSR)) == -1) {
+	if ((out =
+	     open(filename, O_WRONLY | O_CREAT | O_TRUNC,
+		  S_IRUSR | S_IWUSR)) == -1) {
 		ERR(sh, "Could not open %s for writing.", filename);
 		return -1;
 	}
@@ -379,8 +400,9 @@ static int write_file(semanage_handle_t *sh,
 /* Writes a module (or a base) to the file given by a fully-qualified
  * 'filename'.	Returns 0 on success, -1 if file could not be written.
  */
-static int semanage_write_module(semanage_handle_t *sh,
-				 const char *filename, sepol_module_package_t *package)
+static int semanage_write_module(semanage_handle_t * sh,
+				 const char *filename,
+				 sepol_module_package_t * package)
 {
 	struct sepol_policy_file *pf;
 	FILE *outfile;
@@ -412,33 +434,36 @@ static int semanage_write_module(semanage_handle_t *sh,
 /* Commits all changes in sandbox to the actual kernel policy.
  * Returns commit number on success, -1 on error.
  */
-static int semanage_direct_commit(semanage_handle_t *sh) {
+static int semanage_direct_commit(semanage_handle_t * sh)
+{
 	char **mod_filenames = NULL;
+	char *sorted_fc_buffer = NULL, *sorted_nc_buffer = NULL;
+	size_t sorted_fc_buffer_len = 0, sorted_nc_buffer_len = 0;
 	const char *linked_filename = NULL, *ofilename = NULL;
 	sepol_module_package_t *base = NULL;
 	int retval = -1, num_modfiles = 0, i;
-	sepol_policydb_t* out = NULL;
+	sepol_policydb_t *out = NULL;
 
 	/* Declare some variables */
-	int modified, fcontexts_modified, ports_modified, 
-		seusers_modified, users_extra_modified;
-	dbase_config_t* users = semanage_user_dbase_local(sh);
-	dbase_config_t* users_base = semanage_user_base_dbase_local(sh);
-	dbase_config_t* pusers_base = semanage_user_base_dbase_policy(sh);
-	dbase_config_t* users_extra = semanage_user_extra_dbase_local(sh);
-	dbase_config_t* pusers_extra = semanage_user_extra_dbase_policy(sh);
-	dbase_config_t* ports = semanage_port_dbase_local(sh);
-	dbase_config_t* pports = semanage_port_dbase_policy(sh);
-	dbase_config_t* bools = semanage_bool_dbase_local(sh);
-	dbase_config_t* pbools = semanage_bool_dbase_policy(sh);
-	dbase_config_t* ifaces = semanage_iface_dbase_local(sh);
-	dbase_config_t* pifaces = semanage_iface_dbase_policy(sh);
-	dbase_config_t* nodes = semanage_node_dbase_local(sh);
-	dbase_config_t* pnodes = semanage_node_dbase_policy(sh);
-	dbase_config_t* fcontexts = semanage_fcontext_dbase_local(sh);
-	dbase_config_t* pfcontexts = semanage_fcontext_dbase_policy(sh);
-	dbase_config_t* seusers = semanage_seuser_dbase_local(sh);
-	dbase_config_t* pseusers = semanage_seuser_dbase_policy(sh);
+	int modified, fcontexts_modified, ports_modified,
+	    seusers_modified, users_extra_modified;
+	dbase_config_t *users = semanage_user_dbase_local(sh);
+	dbase_config_t *users_base = semanage_user_base_dbase_local(sh);
+	dbase_config_t *pusers_base = semanage_user_base_dbase_policy(sh);
+	dbase_config_t *users_extra = semanage_user_extra_dbase_local(sh);
+	dbase_config_t *pusers_extra = semanage_user_extra_dbase_policy(sh);
+	dbase_config_t *ports = semanage_port_dbase_local(sh);
+	dbase_config_t *pports = semanage_port_dbase_policy(sh);
+	dbase_config_t *bools = semanage_bool_dbase_local(sh);
+	dbase_config_t *pbools = semanage_bool_dbase_policy(sh);
+	dbase_config_t *ifaces = semanage_iface_dbase_local(sh);
+	dbase_config_t *pifaces = semanage_iface_dbase_policy(sh);
+	dbase_config_t *nodes = semanage_node_dbase_local(sh);
+	dbase_config_t *pnodes = semanage_node_dbase_policy(sh);
+	dbase_config_t *fcontexts = semanage_fcontext_dbase_local(sh);
+	dbase_config_t *pfcontexts = semanage_fcontext_dbase_policy(sh);
+	dbase_config_t *seusers = semanage_seuser_dbase_local(sh);
+	dbase_config_t *pseusers = semanage_seuser_dbase_policy(sh);
 
 	/* Before we do anything else, flush the join to its component parts.
 	 * This *does not* flush to disk automatically */
@@ -449,7 +474,8 @@ static int semanage_direct_commit(semanage_handle_t *sh) {
 	/* Decide if anything was modified */
 	fcontexts_modified = fcontexts->dtable->is_modified(fcontexts->dbase);
 	seusers_modified = seusers->dtable->is_modified(seusers->dbase);
-	users_extra_modified = users_extra->dtable->is_modified(users_extra->dbase);
+	users_extra_modified =
+	    users_extra->dtable->is_modified(users_extra->dbase);
 	ports_modified = ports->dtable->is_modified(ports->dbase);
 
 	modified = sh->modules_modified;
@@ -471,38 +497,59 @@ static int semanage_direct_commit(semanage_handle_t *sh) {
 		/* =================== Module expansion =============== */
 
 		/* link all modules in the sandbox to the base module */
-		if (semanage_get_modules_names(sh, &mod_filenames, &num_modfiles) != 0 ||
-		    semanage_verify_modules(sh, mod_filenames, num_modfiles) == -1 ||
-		    semanage_link_sandbox(sh, &base) < 0) {
+		if (semanage_get_modules_names
+		    (sh, &mod_filenames, &num_modfiles) != 0
+		    || semanage_verify_modules(sh, mod_filenames,
+					       num_modfiles) == -1
+		    || semanage_link_sandbox(sh, &base) < 0) {
 			goto cleanup;
 		}
-	
+
 		/* write the linked base */
-		if ((linked_filename = semanage_path(SEMANAGE_TMP, SEMANAGE_LINKED)) == NULL ||
-		    semanage_write_module(sh, linked_filename, base) == -1 ||
-		    semanage_verify_linked(sh) != 0) {
+		if ((linked_filename =
+		     semanage_path(SEMANAGE_TMP, SEMANAGE_LINKED)) == NULL
+		    || semanage_write_module(sh, linked_filename, base) == -1
+		    || semanage_verify_linked(sh) != 0) {
 			goto cleanup;
 		}
 
 		/* ==================== File-backed ================== */
 
 		/* File Contexts */
-		if ((ofilename = semanage_path(SEMANAGE_TMP, SEMANAGE_FC_TMPL)) == NULL ||
-		    write_file(sh, ofilename, sepol_module_package_get_file_contexts(base), 
-				sepol_module_package_get_file_contexts_len(base)) == -1) {
+		/* Sort the file contexts. */
+		if (semanage_fc_sort
+		    (sh, sepol_module_package_get_file_contexts(base),
+		     sepol_module_package_get_file_contexts_len(base),
+		     &sorted_fc_buffer, &sorted_fc_buffer_len) == -1) {
 			goto cleanup;
 		}
 
-		if (semanage_split_fc(sh)) 
+		/* Write the contexts (including template contexts) to a single file.  
+		 * The buffer returned by the sort function has a trailing \0 character,
+		 * which we do NOT want to write out to disk, so we pass sorted_fc_buffer_len-1. */
+		if ((ofilename =
+		     semanage_path(SEMANAGE_TMP, SEMANAGE_FC_TMPL)) == NULL
+		    || write_file(sh, ofilename, sorted_fc_buffer,
+				  sorted_fc_buffer_len - 1) == -1) {
+			goto cleanup;
+		}
+
+		/* Split complete and template file contexts into their separate files. */
+		if (semanage_split_fc(sh))
 			goto cleanup;
 
 		pfcontexts->dtable->drop_cache(pfcontexts->dbase);
 
 		/* Seusers */
 		if (sepol_module_package_get_seusers_len(base)) {
-			if ((ofilename = semanage_path(SEMANAGE_TMP, SEMANAGE_SEUSERS)) == NULL ||
-			    write_file(sh, ofilename, sepol_module_package_get_seusers(base), 
-					sepol_module_package_get_seusers_len(base)) == -1) {
+			if ((ofilename =
+			     semanage_path(SEMANAGE_TMP,
+					   SEMANAGE_SEUSERS)) == NULL
+			    || write_file(sh, ofilename,
+					  sepol_module_package_get_seusers
+					  (base),
+					  sepol_module_package_get_seusers_len
+					  (base)) == -1) {
 				goto cleanup;
 			}
 			pseusers->dtable->drop_cache(pseusers->dbase);
@@ -514,18 +561,44 @@ static int semanage_direct_commit(semanage_handle_t *sh) {
 
 		/* Users_extra */
 		if (sepol_module_package_get_user_extra_len(base)) {
-			if ((ofilename = semanage_path(SEMANAGE_TMP, SEMANAGE_USERS_EXTRA)) == NULL ||
-			    write_file(sh, ofilename, sepol_module_package_get_user_extra(base), 
-					sepol_module_package_get_user_extra_len(base)) == -1) {
+			if ((ofilename =
+			     semanage_path(SEMANAGE_TMP,
+					   SEMANAGE_USERS_EXTRA)) == NULL
+			    || write_file(sh, ofilename,
+					  sepol_module_package_get_user_extra
+					  (base),
+					  sepol_module_package_get_user_extra_len
+					  (base)) == -1) {
 				goto cleanup;
 			}
 			pusers_extra->dtable->drop_cache(pusers_extra->dbase);
 
 		} else {
-			if (pusers_extra->dtable->clear(sh, pusers_extra->dbase) < 0)
+			if (pusers_extra->dtable->
+			    clear(sh, pusers_extra->dbase) < 0)
 				goto cleanup;
 		}
 
+		/* Netfilter Contexts */
+		/* Sort the netfilter contexts. */
+		retval = semanage_nc_sort
+		    (sh, sepol_module_package_get_netfilter_contexts(base),
+		     sepol_module_package_get_netfilter_contexts_len(base),
+		     &sorted_nc_buffer, &sorted_nc_buffer_len);
+
+		if (retval < 0)
+			goto cleanup;
+
+		/* Write the contexts to a single file.  The buffer returned by
+		 * the sort function has a trailing \0 character, which we do
+		 * NOT want to write out to disk, so we pass sorted_fc_buffer_len-1. */
+		ofilename = semanage_path(SEMANAGE_TMP, SEMANAGE_NC);
+		retval = write_file
+		    (sh, ofilename, sorted_nc_buffer, sorted_nc_buffer_len - 1);
+
+		if (retval < 0)
+			goto cleanup;
+
 		/* ==================== Policydb-backed ================ */
 
 		/* Create new policy object, then attach to policy databases
@@ -533,11 +606,12 @@ static int semanage_direct_commit(semanage_handle_t *sh) {
 		if (semanage_expand_sandbox(sh, base, &out) < 0)
 			goto cleanup;
 
-		dbase_policydb_attach((dbase_policydb_t*) pusers_base->dbase, out);
-		dbase_policydb_attach((dbase_policydb_t*) pports->dbase, out);
-		dbase_policydb_attach((dbase_policydb_t*) pifaces->dbase, out);
-		dbase_policydb_attach((dbase_policydb_t*) pbools->dbase, out);
-		dbase_policydb_attach((dbase_policydb_t*) pnodes->dbase, out);
+		dbase_policydb_attach((dbase_policydb_t *) pusers_base->dbase,
+				      out);
+		dbase_policydb_attach((dbase_policydb_t *) pports->dbase, out);
+		dbase_policydb_attach((dbase_policydb_t *) pifaces->dbase, out);
+		dbase_policydb_attach((dbase_policydb_t *) pbools->dbase, out);
+		dbase_policydb_attach((dbase_policydb_t *) pnodes->dbase, out);
 
 		/* ============= Apply changes, and verify  =============== */
 
@@ -567,7 +641,7 @@ static int semanage_direct_commit(semanage_handle_t *sh) {
 
 	/* Validate local seusers against policy */
 	if (sh->do_rebuild || modified || seusers_modified) {
-		if (semanage_seuser_validate_local(sh, out) < 0) 
+		if (semanage_seuser_validate_local(sh, out) < 0)
 			goto cleanup;
 	}
 
@@ -585,50 +659,55 @@ static int semanage_direct_commit(semanage_handle_t *sh) {
 
 	retval = semanage_install_sandbox(sh);
 
- cleanup:
+      cleanup:
 	for (i = 0; mod_filenames != NULL && i < num_modfiles; i++) {
 		free(mod_filenames[i]);
 	}
 
 	/* Detach from policydb, so it can be freed */
-	dbase_policydb_detach((dbase_policydb_t*) pusers_base->dbase);
-	dbase_policydb_detach((dbase_policydb_t*) pports->dbase);
-	dbase_policydb_detach((dbase_policydb_t*) pifaces->dbase);
-	dbase_policydb_detach((dbase_policydb_t*) pnodes->dbase);
-	dbase_policydb_detach((dbase_policydb_t*) pbools->dbase);
+	dbase_policydb_detach((dbase_policydb_t *) pusers_base->dbase);
+	dbase_policydb_detach((dbase_policydb_t *) pports->dbase);
+	dbase_policydb_detach((dbase_policydb_t *) pifaces->dbase);
+	dbase_policydb_detach((dbase_policydb_t *) pnodes->dbase);
+	dbase_policydb_detach((dbase_policydb_t *) pbools->dbase);
 
 	free(mod_filenames);
 	sepol_module_package_free(base);
 	sepol_policydb_free(out);
 	semanage_release_trans_lock(sh);
 
+	free(sorted_fc_buffer);
+	free(sorted_nc_buffer);
+
 	/* regardless if the commit was successful or not, remove the
 	   sandbox if it is still there */
-	semanage_remove_directory(semanage_path(SEMANAGE_TMP, SEMANAGE_TOPLEVEL));
+	semanage_remove_directory(semanage_path
+				  (SEMANAGE_TMP, SEMANAGE_TOPLEVEL));
 	return retval;
 }
 
-
 /* Writes a module to the sandbox's module directory, overwriting any
  * previous module stored within.  Note that module data are not
  * free()d by this function; caller is responsible for deallocating it
  * if necessary.  Returns 0 on success, -1 if out of memory, -2 if the
  * data does not represent a valid module file, -3 if error while
  * writing file. */
-static int semanage_direct_install(semanage_handle_t *sh,
-				   char *data, size_t data_len) {
-	
+static int semanage_direct_install(semanage_handle_t * sh,
+				   char *data, size_t data_len)
+{
+
 	int retval;
 	char *module_name = NULL, *version = NULL, *filename = NULL;
 	if ((retval = parse_module_headers(sh, data, data_len,
-					   &module_name, &version, &filename)) != 0) {
+					   &module_name, &version,
+					   &filename)) != 0) {
 		goto cleanup;
 	}
 	if (write_file(sh, filename, data, data_len) == -1) {
 		retval = -3;
 	}
 	retval = 0;
- cleanup:
+      cleanup:
 	free(version);
 	free(filename);
 	free(module_name);
@@ -643,13 +722,15 @@ static int semanage_direct_install(semanage_handle_t *sh,
  * modules directory, -4 if there does not exist an older module or if
  * the previous module is same or newer than 'data'.
  */
-static int semanage_direct_upgrade(semanage_handle_t *sh,
-				   char *data, size_t data_len) {
+static int semanage_direct_upgrade(semanage_handle_t * sh,
+				   char *data, size_t data_len)
+{
 	int i, retval, num_modules = 0;
 	char *module_name = NULL, *version = NULL, *filename = NULL;
 	semanage_module_info_t *modinfo = NULL;
 	if ((retval = parse_module_headers(sh, data, data_len,
-					   &module_name, &version, &filename)) != 0) {
+					   &module_name, &version,
+					   &filename)) != 0) {
 		goto cleanup;
 	}
 	if (semanage_direct_list(sh, &modinfo, &num_modules) < 0) {
@@ -657,32 +738,36 @@ static int semanage_direct_upgrade(semanage_handle_t *sh,
 	}
 	retval = -4;
 	for (i = 0; i < num_modules; i++) {
-		semanage_module_info_t *m = semanage_module_list_nth(modinfo, i);
+		semanage_module_info_t *m =
+		    semanage_module_list_nth(modinfo, i);
 		if (strcmp(semanage_module_get_name(m), module_name) == 0) {
-			if (strverscmp(version, semanage_module_get_version(m)) > 0) {
+			if (strverscmp(version, semanage_module_get_version(m))
+			    > 0) {
 				retval = 0;
 				break;
-			}
-			else {
-				ERR(sh, "Previous module %s is same or newer.", module_name);
+			} else {
+				ERR(sh, "Previous module %s is same or newer.",
+				    module_name);
 				retval = -4;
 				goto cleanup;
 			}
 		}
 	}
 	if (retval == -4) {
-		ERR(sh, "There does not already exist a module named %s.", module_name);
+		ERR(sh, "There does not already exist a module named %s.",
+		    module_name);
 		goto cleanup;
 	}
 	if (write_file(sh, filename, data, data_len) == -1) {
 		retval = -3;
 	}
- cleanup:
+      cleanup:
 	free(version);
 	free(filename);
 	free(module_name);
 	for (i = 0; modinfo != NULL && i < num_modules; i++) {
-		semanage_module_info_t *m = semanage_module_list_nth(modinfo, i);
+		semanage_module_info_t *m =
+		    semanage_module_list_nth(modinfo, i);
 		semanage_module_info_datum_destroy(m);
 	}
 	free(modinfo);
@@ -695,8 +780,9 @@ static int semanage_direct_upgrade(semanage_handle_t *sh,
  * on success, -1 if out of memory, -2 if the data does not represent
  * a valid base module file, -3 if error while writing file.
  */
-static int semanage_direct_install_base(semanage_handle_t *sh,
-					char *base_data, size_t data_len) {
+static int semanage_direct_install_base(semanage_handle_t * sh,
+					char *base_data, size_t data_len)
+{
 	int retval = -1;
 	const char *filename = NULL;
 	if ((retval = parse_base_headers(sh, base_data, data_len)) != 0) {
@@ -709,19 +795,20 @@ static int semanage_direct_install_base(semanage_handle_t *sh,
 		retval = -3;
 	}
 	retval = 0;
- cleanup:
+      cleanup:
 	return retval;
 }
 
 /* Removes a module from the sandbox.  Returns 0 on success, -1 if out
  * of memory, -2 if module not found or could not be removed. */
-static int semanage_direct_remove(semanage_handle_t *sh,
-				  char *module_name) {
+static int semanage_direct_remove(semanage_handle_t * sh, char *module_name)
+{
 	int i, retval = -1;
 	char **module_filenames = NULL;
 	int num_mod_files;
 	size_t name_len = strlen(module_name);
-	if (semanage_get_modules_names(sh, &module_filenames, &num_mod_files) == -1) {
+	if (semanage_get_modules_names(sh, &module_filenames, &num_mod_files) ==
+	    -1) {
 		return -1;
 	}
 	for (i = 0; i < num_mod_files; i++) {
@@ -735,7 +822,8 @@ static int semanage_direct_remove(semanage_handle_t *sh,
 		if (memcmp(module_name, base, name_len) == 0 &&
 		    strcmp(base + name_len, ".pp") == 0) {
 			if (unlink(module_filenames[i]) == -1) {
-				ERR(sh, "Could not remove module file %s.", module_filenames[i]);
+				ERR(sh, "Could not remove module file %s.",
+				    module_filenames[i]);
 				retval = -2;
 			}
 			retval = 0;
@@ -743,8 +831,8 @@ static int semanage_direct_remove(semanage_handle_t *sh,
 		}
 	}
 	ERR(sh, "Module %s was not found.", module_name);
-	retval = -2;				/* module not found */
- cleanup:
+	retval = -2;		/* module not found */
+      cleanup:
 	for (i = 0; module_filenames != NULL && i < num_mod_files; i++) {
 		free(module_filenames[i]);
 	}
@@ -752,7 +840,6 @@ static int semanage_direct_remove(semanage_handle_t *sh,
 	return retval;
 }
 
-
 /* Allocate an array of module_info structures for each readable
  * module within the store.  Note that if the calling program has
  * already begun a transaction then this function will get a list of
@@ -760,8 +847,10 @@ static int semanage_direct_remove(semanage_handle_t *sh,
  * semanage_module_info_datum_destroy() on each element of the array
  * as well as free()ing the entire list.
  */
-static int semanage_direct_list(semanage_handle_t *sh,
-				semanage_module_info_t **modinfo, int *num_modules) {
+static int semanage_direct_list(semanage_handle_t * sh,
+				semanage_module_info_t ** modinfo,
+				int *num_modules)
+{
 	struct sepol_policy_file *pf = NULL;
 	int i, retval = -1;
 	char **module_filenames = NULL;
@@ -769,13 +858,14 @@ static int semanage_direct_list(semanage_handle_t *sh,
 	*modinfo = NULL;
 	*num_modules = 0;
 
-        /* get the read lock when reading from the active
-           (non-transaction) directory */
-	if (!sh->is_in_transaction) 
-		if (semanage_get_active_lock(sh) < 0) 
+	/* get the read lock when reading from the active
+	   (non-transaction) directory */
+	if (!sh->is_in_transaction)
+		if (semanage_get_active_lock(sh) < 0)
 			return -1;
 
-	if (semanage_get_modules_names(sh, &module_filenames, &num_mod_files) == -1) {
+	if (semanage_get_modules_names(sh, &module_filenames, &num_mod_files) ==
+	    -1) {
 		goto cleanup;
 	}
 	if (num_mod_files == 0) {
@@ -788,12 +878,12 @@ static int semanage_direct_list(semanage_handle_t *sh,
 		goto cleanup;
 	}
 	sepol_policy_file_set_handle(pf, sh->sepolh);
-	
+
 	if ((*modinfo = calloc(num_mod_files, sizeof(**modinfo))) == NULL) {
 		ERR(sh, "Out of memory!");
 		goto cleanup;
 	}
-	
+
 	for (i = 0; i < num_mod_files; i++) {
 		FILE *fp;
 		char *name = NULL, *version = NULL;
@@ -816,31 +906,32 @@ static int semanage_direct_list(semanage_handle_t *sh,
 			(*modinfo)[*num_modules].name = name;
 			(*modinfo)[*num_modules].version = version;
 			(*num_modules)++;
-		}
-		else {
+		} else {
 			/* file was not a module, so don't report it */
 			free(name);
 			free(version);
 		}
 	}
 	retval = semanage_get_commit_number(sh);
-	
- cleanup:
+
+      cleanup:
 	sepol_policy_file_free(pf);
 	for (i = 0; module_filenames != NULL && i < num_mod_files; i++) {
 		free(module_filenames[i]);
 	}
 	free(module_filenames);
-        if (!sh->is_in_transaction) {
-                semanage_release_active_lock(sh);
-        }
+	if (!sh->is_in_transaction) {
+		semanage_release_active_lock(sh);
+	}
 	return retval;
 }
 
-int semanage_direct_access_check(semanage_handle_t *sh) {
-	char polpath[PATH_MAX];	
+int semanage_direct_access_check(semanage_handle_t * sh)
+{
+	char polpath[PATH_MAX];
 
-	snprintf(polpath, PATH_MAX, "%s%s", selinux_path(), sh->conf->store_path);
+	snprintf(polpath, PATH_MAX, "%s%s", selinux_path(),
+		 sh->conf->store_path);
 
 	if (semanage_check_init(polpath))
 		return -1;
