@@ -33,21 +33,18 @@
 #include "policy.h"
 
 struct semanage_handle {
-	int con_id;             /* Connection ID */
+	int con_id;		/* Connection ID */
 
 	/* Error handling */
 	int msg_level;
-	const char* msg_channel;
-	const char* msg_fname;
+	const char *msg_channel;
+	const char *msg_fname;
 #ifdef __GNUC__
-	__attribute__ ((format (printf, 3, 4)))
+	__attribute__ ((format(printf, 3, 4)))
 #endif
-	void (*msg_callback) (
-		void* varg,
-		semanage_handle_t* handle,
-		const char* fmt,
-		...);
-	void* msg_callback_arg;
+	void (*msg_callback) (void *varg,
+			      semanage_handle_t * handle, const char *fmt, ...);
+	void *msg_callback_arg;
 
 	/* Direct vs Server specific handle */
 	union {
@@ -55,7 +52,7 @@ struct semanage_handle {
 	} u;
 
 	/* Libsepol handle */
-	sepol_handle_t* sepolh;
+	sepol_handle_t *sepolh;
 
 	semanage_conf_t *conf;
 	int is_connected;
@@ -68,14 +65,14 @@ struct semanage_handle {
 
 	/* This timeout is used for transactions and waiting for lock
 	   -1 means wait indefinetely
-	    0 means return immediately
-	    >0 means wait that many seconds */
+	   0 means return immediately
+	   >0 means wait that many seconds */
 	int timeout;
 
 	/* these function pointers will point to the appropriate
 	 * routine given the connection type.  think of these as
 	 * simulating polymorphism for non-OO languages. */
-	struct semanage_policy_table* funcs;
+	struct semanage_policy_table *funcs;
 
 	/* Object databases */
 #define DBASE_COUNT      19
@@ -109,102 +106,120 @@ struct semanage_handle {
 
 /* === Local modifications === */
 static inline
-dbase_config_t* semanage_user_base_dbase_local(semanage_handle_t* handle) {
+    dbase_config_t * semanage_user_base_dbase_local(semanage_handle_t * handle)
+{
 	return &handle->dbase[DBASE_LOCAL_USERS_BASE];
 }
 
 static inline
-dbase_config_t* semanage_user_extra_dbase_local(semanage_handle_t* handle) {
+    dbase_config_t * semanage_user_extra_dbase_local(semanage_handle_t * handle)
+{
 	return &handle->dbase[DBASE_LOCAL_USERS_EXTRA];
 }
 
-static inline 
-dbase_config_t* semanage_user_dbase_local(semanage_handle_t* handle) {
+static inline
+    dbase_config_t * semanage_user_dbase_local(semanage_handle_t * handle)
+{
 	return &handle->dbase[DBASE_LOCAL_USERS];
 }
 
-static inline 
-dbase_config_t* semanage_port_dbase_local(semanage_handle_t* handle) {
+static inline
+    dbase_config_t * semanage_port_dbase_local(semanage_handle_t * handle)
+{
 	return &handle->dbase[DBASE_LOCAL_PORTS];
 }
 
 static inline
-dbase_config_t* semanage_iface_dbase_local(semanage_handle_t* handle) {
+    dbase_config_t * semanage_iface_dbase_local(semanage_handle_t * handle)
+{
 	return &handle->dbase[DBASE_LOCAL_INTERFACES];
 }
 
 static inline
-dbase_config_t* semanage_bool_dbase_local(semanage_handle_t* handle) {
+    dbase_config_t * semanage_bool_dbase_local(semanage_handle_t * handle)
+{
 	return &handle->dbase[DBASE_LOCAL_BOOLEANS];
 }
 
 static inline
-dbase_config_t* semanage_fcontext_dbase_local(semanage_handle_t* handle) {
+    dbase_config_t * semanage_fcontext_dbase_local(semanage_handle_t * handle)
+{
 	return &handle->dbase[DBASE_LOCAL_FCONTEXTS];
 }
 
 static inline
-dbase_config_t* semanage_seuser_dbase_local(semanage_handle_t* handle) {
+    dbase_config_t * semanage_seuser_dbase_local(semanage_handle_t * handle)
+{
 	return &handle->dbase[DBASE_LOCAL_SEUSERS];
 }
 
 static inline
-dbase_config_t* semanage_node_dbase_local(semanage_handle_t* handle) {
+    dbase_config_t * semanage_node_dbase_local(semanage_handle_t * handle)
+{
 	return &handle->dbase[DBASE_LOCAL_NODES];
 }
 
-
 /* === Policy + Local modifications === */
 static inline
-dbase_config_t* semanage_user_base_dbase_policy(semanage_handle_t* handle) {
+    dbase_config_t * semanage_user_base_dbase_policy(semanage_handle_t * handle)
+{
 	return &handle->dbase[DBASE_POLICY_USERS_BASE];
 }
 
 static inline
-dbase_config_t* semanage_user_extra_dbase_policy(semanage_handle_t* handle) {
+    dbase_config_t * semanage_user_extra_dbase_policy(semanage_handle_t *
+						      handle)
+{
 	return &handle->dbase[DBASE_POLICY_USERS_EXTRA];
 }
 
 static inline
-dbase_config_t* semanage_user_dbase_policy(semanage_handle_t* handle) {
+    dbase_config_t * semanage_user_dbase_policy(semanage_handle_t * handle)
+{
 	return &handle->dbase[DBASE_POLICY_USERS];
 }
 
 static inline
-dbase_config_t* semanage_port_dbase_policy(semanage_handle_t* handle) {
+    dbase_config_t * semanage_port_dbase_policy(semanage_handle_t * handle)
+{
 	return &handle->dbase[DBASE_POLICY_PORTS];
 }
 
 static inline
-dbase_config_t* semanage_iface_dbase_policy(semanage_handle_t* handle) {
+    dbase_config_t * semanage_iface_dbase_policy(semanage_handle_t * handle)
+{
 	return &handle->dbase[DBASE_POLICY_INTERFACES];
 }
 
 static inline
-dbase_config_t* semanage_bool_dbase_policy(semanage_handle_t* handle) {
+    dbase_config_t * semanage_bool_dbase_policy(semanage_handle_t * handle)
+{
 	return &handle->dbase[DBASE_POLICY_BOOLEANS];
 }
 
 static inline
-dbase_config_t* semanage_fcontext_dbase_policy(semanage_handle_t* handle) {
+    dbase_config_t * semanage_fcontext_dbase_policy(semanage_handle_t * handle)
+{
 	return &handle->dbase[DBASE_POLICY_FCONTEXTS];
 }
 
 static inline
-dbase_config_t* semanage_seuser_dbase_policy(semanage_handle_t* handle) {
+    dbase_config_t * semanage_seuser_dbase_policy(semanage_handle_t * handle)
+{
 	return &handle->dbase[DBASE_POLICY_SEUSERS];
 }
 
 static inline
-dbase_config_t* semanage_node_dbase_policy(semanage_handle_t* handle) {
+    dbase_config_t * semanage_node_dbase_policy(semanage_handle_t * handle)
+{
 	return &handle->dbase[DBASE_POLICY_NODES];
 }
 
 /* === Active kernel policy === */
 static inline
-dbase_config_t* semanage_bool_dbase_active(semanage_handle_t* handle) {
+    dbase_config_t * semanage_bool_dbase_active(semanage_handle_t * handle)
+{
 	return &handle->dbase[DBASE_ACTIVE_BOOLEANS];
 }
 
 #endif
-
