@@ -1,8 +1,9 @@
 /* Authors: Karl MacMillan <kmacmillan@tresys.com>
  *	    Joshua Brindle <jbrindle@tresys.com>
  *	    Jason Tang <jtang@tresys.com>
+ *	    Christopher Ashworth <cashworth@tresys.com>
  *
- * Copyright (C) 2004-2005 Tresys Technology, LLC
+ * Copyright (C) 2004-2006 Tresys Technology, LLC
  * Copyright (C) 2005 Red Hat, Inc.
  *
  *  This library is free software; you can redistribute it and/or
@@ -37,10 +38,10 @@ enum semanage_store_defs {
 /* sandbox filenames and paths */
 enum semanage_sandbox_defs {
 	SEMANAGE_TOPLEVEL,
-	SEMANAGE_MODULES, 
+	SEMANAGE_MODULES,
 	SEMANAGE_KERNEL,
-	SEMANAGE_BASE, 
-	SEMANAGE_LINKED, 
+	SEMANAGE_BASE,
+	SEMANAGE_LINKED,
 	SEMANAGE_FC,
 	SEMANAGE_HOMEDIR_TMPL,
 	SEMANAGE_FC_TMPL,
@@ -55,6 +56,7 @@ enum semanage_sandbox_defs {
 	SEMANAGE_USERS_EXTRA_LOCAL,
 	SEMANAGE_SEUSERS,
 	SEMANAGE_USERS_EXTRA,
+	SEMANAGE_NC,
 	SEMANAGE_STORE_NUM_PATHS
 };
 
@@ -64,57 +66,60 @@ enum semanage_sandbox_defs {
 const char *semanage_conf_path(void);
 int semanage_check_init(const char *root);
 
-extern const char* semanage_fname(
-	enum semanage_sandbox_defs file_enum);
+extern const char *semanage_fname(enum semanage_sandbox_defs file_enum);
 
-extern const char *semanage_path(
-	enum semanage_store_defs store, 
-	enum semanage_sandbox_defs file);
+extern const char *semanage_path(enum semanage_store_defs store,
+				 enum semanage_sandbox_defs file);
 
-int semanage_create_store(semanage_handle_t *sh, int create);
+int semanage_create_store(semanage_handle_t * sh, int create);
 
-int semanage_store_access_check(semanage_handle_t *sh);
+int semanage_store_access_check(semanage_handle_t * sh);
 
 int semanage_remove_directory(const char *path);
 
-int semanage_make_sandbox(semanage_handle_t *sh);
+int semanage_make_sandbox(semanage_handle_t * sh);
 
-int semanage_get_modules_names(
-	semanage_handle_t *sh,
-	char ***filenames, 
-	int *len);
+int semanage_get_modules_names(semanage_handle_t * sh,
+			       char ***filenames, int *len);
 
-int semanage_install_sandbox(semanage_handle_t *sh);
+int semanage_install_sandbox(semanage_handle_t * sh);
 
 /* lock file routines */
-int semanage_get_trans_lock(semanage_handle_t *sh);
-int semanage_get_active_lock(semanage_handle_t *sh);
-void semanage_release_trans_lock(semanage_handle_t *sh);
-void semanage_release_active_lock(semanage_handle_t *sh);
-int semanage_get_commit_number(semanage_handle_t *sh);
+int semanage_get_trans_lock(semanage_handle_t * sh);
+int semanage_get_active_lock(semanage_handle_t * sh);
+void semanage_release_trans_lock(semanage_handle_t * sh);
+void semanage_release_active_lock(semanage_handle_t * sh);
+int semanage_get_commit_number(semanage_handle_t * sh);
 
-int semanage_link_sandbox(
-	semanage_handle_t *sh, 
-	sepol_module_package_t **base);
+int semanage_link_sandbox(semanage_handle_t * sh,
+			  sepol_module_package_t ** base);
 
-int semanage_expand_sandbox(
-	semanage_handle_t *sh, 
-	sepol_module_package_t *base,
-	sepol_policydb_t** policydb);
+int semanage_expand_sandbox(semanage_handle_t * sh,
+			    sepol_module_package_t * base,
+			    sepol_policydb_t ** policydb);
 
-int semanage_write_policydb(
-	semanage_handle_t *sh,
-	sepol_policydb_t* policydb);
+int semanage_write_policydb(semanage_handle_t * sh,
+			    sepol_policydb_t * policydb);
 
-int semanage_install_sandbox(semanage_handle_t *sh);
+int semanage_install_sandbox(semanage_handle_t * sh);
 
-int semanage_verify_modules(
-	semanage_handle_t *sh,
-	char **module_filenames, 
-	int num_modules);
+int semanage_verify_modules(semanage_handle_t * sh,
+			    char **module_filenames, int num_modules);
 
-int semanage_verify_linked(semanage_handle_t *sh);
-int semanage_verify_kernel(semanage_handle_t *sh);
-int semanage_split_fc(semanage_handle_t *sh);
+int semanage_verify_linked(semanage_handle_t * sh);
+int semanage_verify_kernel(semanage_handle_t * sh);
+int semanage_split_fc(semanage_handle_t * sh);
+
+/* sort file context routines */
+int semanage_fc_sort(semanage_handle_t * sh,
+		     const char *buf,
+		     size_t buf_len,
+		     char **sorted_buf, size_t * sorted_buf_len);
+
+/* sort netfilter context routines */
+int semanage_nc_sort(semanage_handle_t * sh,
+		     const char *buf,
+		     size_t buf_len,
+		     char **sorted_buf, size_t * sorted_buf_len);
 
 #endif
